require 'lib/mailchimp_client'

namespace :mailchimp do
  def mailchimp_client
    return @mailchimp_client if @mailchimp_client
    @mailchimp_client = MailchimpClient.new(MAILCHIMP[:api_key])
    @mailchimp_client.use_list(MAILCHIMP[:list_name])
    @mailchimp_client
  end

  desc "Add existing mailing list subscribers to Mailchimp"
  task :batch_subscribe => [:environment] do
    batch = User.find(:all, :conditions => ["opt_in_products = 1 OR opt_in_promotions = 1"]).collect {|user| user.to_mailchimp}
    puts "Adding #{batch.count} emails to the '#{MAILCHIMP[:list_name]}' list..."

    mailchimp_client.batch_subscribe(batch)

    puts "Received this response from Mailchimp: #{mailchimp_client.previous_response}"
    puts "Done!"
  end

  desc "Remove subscriber data on the Mailchimp server"
  task :destroy_subscribers => [:environment] do
    puts "WARNING: This will destroy subscriber data on the Mailchimp server"

    after_confirmation do
      mailchimp_client.destroy_all_subscribers!
      puts "List now has #{mailchimp_client.subscribers.count} subscribers."
      puts "Done!"
    end
  end

  def after_confirmation(&block)
    while true
      print "  Are you sure you want to proceed? [Y/N] "
      response = STDIN.gets
      if response =~ /^[yY]/
        puts
        block.call()
        return
      elsif response =~ /^[nN]/
        puts "  Terminating early as requested."
        return
      end
    end
  end
end
